{-# LANGUAGE CPP #-}
{-
 Copyright (c) 2005 Jesper Louis Andersen <jlouis@mongers.org>

 Permission to use, copy, modify, and distribute this software for any
 purpose with or without fee is hereby granted, provided that the above
 copyright notice and this permission notice appear in all copies.

 THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Conjure.Version
-- Copyright   :  (c) Jesper Louis Andersen, 2005
-- License     :  BSD-style (see the file libraries/base/LICENSE)
--
-- Maintainer  :  jlouis@mongers.org
-- Stability   :  believed to be stable
-- Portability :  portable
--
-- The Conjure.Version module is responsible for Conjure Version
-- control.
-----------------------------------------------------------------------------
#ifndef __HADDOCK__
#include <ghcconfig.h>
#endif
module Conjure.Version
  (
    conjure_version,
    showVersionHeader
  )
where

import Data.Version as Version

-- | List the conjure version. Our version numbers are listed by
--   Date, like all versions of software should be. To make it easier
--   to refer to a given version, they are given a tag which is a
--   pronounceable string with a certain magical feel to it.
conjure_version :: Version.Version
conjure_version =
  Version.Version { Version.versionBranch = [2005, 12, 15],
                    Version.versionTags = ["WASTING"]}

-- Other names for different versionTags:
-- SORCERY, MAGIC, RITUAL, RITE, CAULDRON, ILLUSION, DEMONOLOGY,
-- NECROMANCY, DRAGONOLOGY, ASTRAL, PLANEWALKER, FIREBALL,
-- LIGHTNING_STRIKE, PENTAGRAM, SOULTAP, BARRIER, SUMMON, HEX,
-- WIZARDRY, CONJURATON, INFLUENCE, DISINTEGRATE, ELEMENTALIST,
-- EXORCISM, TELEKINESIS, TELEPATHY, TREMOR, TRANQUILITY, ALAKAZAM!
-- MANDRAKE, NIGHTSHADE, FOCI, BANISHMENT, WANING, WAXING.
-- etc... (jlouis)

-- Old names: BREWING STEWING

-- | Convert the Version into a header
-- FIXME
showVersionHeader :: Version.Version -> String
showVersionHeader version =
  concat ["This is Conjure built for ", "PLATFORM",
          "\n  GHC version: ", "GHC_VERSION",
          "\n  Version: ", Version.showVersion version,
          "\n  Internal patch count: ", "PATCH_COUNT",
          "\nI am ready to summon the files you need."]
